/* protop.p
 *
 * start interactive protop client
 *
 * mpro -p protop -param "friendlyName|startupDataCollectorList|connectString|options"
 *
 */

do on stop   undo, retry								/* drop out on any untrapped fatal errors			*/
   on error  undo, retry
   on endkey undo, retry:

  if retry then										/* this should never fire (but it does, usually during		*/
    do:											/* development due to compiler errors)				*/
      message
        "Unhandled fatal error:" error-status:get-number(1) error-status:get-message(1)
       view-as
        alert-box information
      .
      quit.
    end.

/* the end of this block is at the end of protop.p... indenting the whole program just seems silly
 */

&IF DECIMAL(SUBSTRING(PROVERSION,1,INDEX(PROVERSION,".") + 1)) > 11.0 &THEN

if os-getenv( "TTDEBUG" ) = "yes" then
  do:
    Progress.Database.TempTableInfo:ArchiveTableStatistics = true no-error.
    Progress.Database.TempTableInfo:ArchiveIndexStatistics = true no-error.
  end.

&ENDIF

assign
  session:date-format    = "ymd"                                			/* the one true way to format dates!				*/
  session:numeric-format = "american"                                			/* a lazy american must have written this code...		*/
.

{lib/protop.i}
{lib/dynscreen.i}

define new global shared variable pt_first as logical no-undo.
pt_first = yes.

define new global shared variable brSP as integer no-undo initial 1.			/* browse spacing						*/

define temp-table tt_protop no-undo
  field order    as integer
  field scrType  as character
  field procName as character
  field scrTitle as character
  field dcName   as character
  field ttName   as character
  field minRows  as integer
  field maxRows  as integer
  field ptCmd    as character
  index pt-idx is primary unique order
.

function OS-GetWinSize returns logical ( input-output numRows as integer, input-output numColumns as integer ) in super.

dbgMode = 1.

define stream initStrm.

define new global shared variable monInt         as integer no-undo initial 10.		/* monitor sampling interval					*/

define new global shared variable dsSize         as integer  no-undo.			/* the shame!							*/
define new global shared variable dsRows         as integer  no-undo.			/* the shame!							*/
define new global shared variable programmerMode as logical  no-undo.

define new global shared variable userTblName as character   no-undo.
define new global shared variable userTblNum  as integer     no-undo initial -999999.
define new global shared variable userIdxName as character   no-undo.
define new global shared variable userIdxNum  as integer     no-undo initial -999999.


define variable fast-start  as logical     no-undo initial yes.				/* if "no" then a pause will affect startup messages		*/

define variable tick        as decimal     no-undo initial 0.5.				/* clock tick - 1/2 sec rounds up to 1 prior to OE11		*/
define variable monName     as character   no-undo initial "protop".
define variable dbgFileName as character   no-undo.					/* echo "3" > tmp/protop.s2k.debug				*/
define variable prfFileName as character   no-undo.					/* echo "on" > tmp/protop.s2k.prf.flg				*/

define variable dcList      as character   no-undo.					/* list of active data collectors				*/
define variable result      as character   no-undo.					/* serialized XML or JSON from data collectors			*/

define variable ptCnxStr    as character   no-undo.					/* how to connect to remote data collectors			*/
define variable ptParams    as character   no-undo.					/* parameters for remote data collectors			*/

define variable ptStartup   as character   no-undo.					/* which data collectors should we start with?			*/
define variable xtraStart   as character   no-undo.					/* extra startup commands (uses env variable, below)		*/

define variable sampleMode  as character   no-undo initial "Auto".			/* are we sampling every X seconds?  or on-demand		*/
define variable connectMode as character   no-undo initial "Direct".			/* direct?  http?  stomp?					*/
define variable prfMode     as character   no-undo.					/* profiler status - on or off					*/

define variable keepRunning as logical     no-undo initial yes.				/* if false then quit ProTop					*/

define variable lastRefresh as integer     no-undo.					/* when was the last refresh?					*/
define variable fetchTime   as integer     no-undo.					/* how long does it take to fetch data?				*/

define variable initReqStr  as character   no-undo.					/* initial data collectors requested				*/

define variable userNum     as integer     no-undo.					/* Usr# for userInfo data collector				*/
define variable userPID     as integer     no-undo.					/* PID for userInfo data collector				*/

define variable demoList    as character   no-undo					/* db logical names that demo code will run with		*/
  initial "sports2000,s2k,bigsports".							/* plain old "sports" is too small and lacks orderDate		*/

define variable dotMode     as integer     no-undo.					/* 0 = off, 1 = on, 2 = sticky					*/
define variable rawMode     as integer     no-undo initial 5.				/* interval (5), cumulative (4) or "raw" (3) metrics		*/
define variable timeMode    as integer     no-undo initial 2.				/* rate (2) or "summary" (1) metrics				*/
define variable CSC         as logical     no-undo initial ?.				/* Client Statement Cache on/off				*/
define variable logMode     as logical     no-undo.					/* control the logging mode					*/

define variable numSamples  as integer     no-undo.					/* how many samples have we gathered?				*/

define variable b as handle  no-undo.							/* a buffer handle						*/
define variable i as integer no-undo.							/* an integer							*/

define variable numRows    as integer   no-undo.					/* window height						*/
define variable numColumns as integer   no-undo.					/* window width							*/

/* needed if the connection is to a remote db via http agents (connectMode = "http")  -- currently unused
 */

define variable url  as character no-undo.						/* "localhost/cgi-bin/protop.cgi?type=pt3http&DB=s2k0&UID=tom"	*/
define variable host as character no-undo.
define variable port as character no-undo.
define variable path as character no-undo.

/* variables for the screen header (row 1): built in procedure paintScreen() below
 */

define variable headerFrame   as handle no-undo.
define variable modeIndicator as handle no-undo.
define variable protopVersion as handle no-undo.
define variable clockString   as handle no-undo.

define variable dataColor     as integer no-undo.
define variable goodColor     as integer no-undo.
define variable alertColor    as integer no-undo.

/* row 2 of the header is built by dc/dbid.p
 */

/* a fake variable and frame that will be positioned on row 3, which is always blank, so as to be invisible.  this
 * gives the READKEY statement a widget for focus which fixes the problems of being able to move browsers with the
 * arrow keys as well as the hiding of a seemingly random browse title.
 *
 * all of which then provides a foundation for perhaps moving pseudo-focus among the frames to permit scrolling
 * once I get some time to mess around with such things.
 *
 */

define variable nothing   as character no-undo format "x".
define variable something as character no-undo format "x(20)".				/* useful for debugging what keys were pressed...		*/

form
  nothing something
 with
  frame notHere
  no-box
  no-labels
  overlay
  row 3
  column 1
  color display normal prompt normal							/* no underline for the invisible field...			*/
.


/* event handlers and internal procedures
 *
 */

on any-printable of this-procedure do:
  message "eh?  that shouldn't have happened!".
  pause.
  publish "protop_command" ( chr( lastkey )).
  return. /* no-apply. */
end.


on window-close anywhere do:
  keepRunning = false.
  return.
end.


on any-printable of nothing in frame notHere do:
  something = something + chr( lastkey ).
  do while length( something ) > 8:
    something = substring( something, 2 ).
  end.
  color display normal something with frame notHere.					/* help debug the vanishing keystroke problem...		*/
  color prompt  normal something with frame notHere.					/* help debug the vanishing keystroke problem...		*/
  display something with frame notHere.							/* help debug the vanishing keystroke problem...		*/
  publish "protop_command" ( chr( lastkey )).
  return. /* no-apply. */
end.


procedure notyet:

  define input parameter c as character no-undo.

  message
    skip(1)
    "The command character:" c " is not yet implemented and has no function."
    skip(1)
   view-as alert-box information title " Not Yet "
  .

  return.

end.


procedure frameInfo:

  define input parameter w as handle no-undo.

  define variable n as handle no-undo.


  if w:type = "frame" and w:visible = yes then
    do:
      message w:name.									/* plus whatever info you might want...				*/
      pause.
    end.

  n = w:first-child no-error.
  if valid-handle( n ) then
    run frameInfo( n ).

  n = w:next-sibling no-error.
  if valid-handle( n ) then
    run frameInfo( n ).

  return.

end.


procedure viewer:

  define variable fileName      as character no-undo.

  run lib/filepicker.p ( pt_rptdir, "*", input-output fileName ).
  file-info:file-name = fileName.
  if file-info:full-pathname <> ? then run lib/fileviewer.p ( fileName ).

  return.

end.


procedure dotMode:

  if dotMode = 0 then
    dotMode = 1.
   else
    dotMode = 0.

  return.

end.


/* ProTop might not be running as a stand-alone program so we do a check
 * of the call stack to make sure that we really should quit.
 */

procedure quitme:									/* If there is no parent procedure, we QUIT, otherwise drop	*/
											/* drop out of the main event loop & RETURN			*/
  keepRunning = FALSE.

  /* reminder to signup for the web portal might go here */

  if session:window-system <> "tty" then						/* prowin?							*/
    apply "window-close" to this-procedure.
   else
    if program-name( 4 ) = ? then QUIT.

  return.

end.


procedure setSampleMode:

  if sampleMode = "OnDemand" then							/* this needs to be visible -- a header line is needed		*/
    sampleMode = "Auto".
   else
    sampleMode = "OnDemand".

  return.

end.


procedure userInfo2:									/* userInfo returns extra data - when it is choosen also	*/
											/* activate these						*/
  publish "protop_usrtblact".
  publish "protop_usridxact".
  publish "protop_usrstack".
  publish "protop_usrother".

  return.

end.


procedure refresh:

  define variable reqString as character no-undo.

  /* request the data collectors which are visible or active (this procedure is in lib/dynscreen.p)
   */
     
  publish "protop_getReqStr" ( input-output reqString ).
  
  /* edit the request string to add arguments to certain data collectors if the are requested
   */

  if lookup( "userInformation", reqString ) > 0 then
    do:

      if userNum <> ? then
        reqString = replace( reqString, "userInformation", substitute( "userInformation:USR=&1", userNum )).
       else if userPID <> ? then
        reqString = replace( reqString, "userInformation", substitute( "userInformation:PID=&1", userPID )).

    end.

  if lookup( "UsersOfTable", reqString ) > 0 then
    do:

      if userTblNum <> ? then
        reqString = replace( reqString, "usersOfTable", substitute( "usersOfTable:UTBLNUM=&1", userTblNum )).

    end.

  if lookup( "UsersOfIndex", reqString ) > 0 then
    do:

      /* message userIdxName userIdxNum replace( reqString, "usersOfIndex", substitute( "usersOfIndex:UIDXNUM=&1", userIdxNum )) view-as alert-box information. */

      if userIdxNum <> ? then
        reqString = replace( reqString, "usersOfIndex", substitute( "usersOfIndex:UIDXNUM=&1", userIdxNum )).

    end.

  if opsys = "unix" then
    do:
      color display red something with frame notHere.					/* something swallows keystrokes when makeRequest() runs	*/
    end.
   else		/* windows insists on a gray background for some reason	- this fixes that but at the cost of no red :( */
    do:
      color prompt  normal something with frame notHere.				/* something swallows keystrokes when makeRequest() runs	*/
      color display normal something with frame notHere.				/* something swallows keystrokes when makeRequest() runs	*/
    end.

  if session:batch = yes then
    message "makeRequest(" reqString ")".

  run makeRequest ( reqString ).

  /* color display blue something with frame notHere. */				/* paintScreen is innocent...					*/

  run paintScreen.

  return.

end.


procedure makeRequest:

  define input parameter reqString as character no-undo.

  fetchTime = etime.									/* fetchStart = now.						*/
 
  if connectMode = "direct" then
    run doRequest( "|" + reqString /* , output result */ ).				/* we don't need userid for a direct connection 		*/

  return.

end.


define variable rt as character no-undo.

procedure paintScreen:

  fetchTime = etime - fetchTime.							/* fetchTime = abs( interval( now, fetchStart, "milliseconds" )). */

  dynScreenUpdate().
  rt = "DS".

  if session:batch = no then
    assign										/* first line of header						*/
      modeIndicator:screen-value = string( today, "9999/99/99" )			/* upper left							*/
      clockString:screen-value   = string( time, "hh:mm:ss" )				/* upper right							*/
      headerFrame:dcolor         = getColorNum( "header" )
      headerFrame:width-chars    = current-window:width-chars
      headerFrame:hidden         = no
      headerFrame:visible        = yes
    .

  {ssg/sausage01.i}

  result = "".

  return.

end.


procedure dbanalys:

  define variable ok as logical no-undo.

  message
    skip(1)
    "  Running a dbanalys can be very IO intensive.  You might prefer to run  " skip
    "  this utility offline during a quiet period.                            " skip
    skip(1)
    "  To run it offline use a command similar to:                            " skip
    skip(1)
    substitute( "  proutil &1 -C dbanalys > dbanalys/&2.dba  ", ldbname(1), ( if pt_shortname <> "" then pt_shortname else ldbname(1) )) skip
    skip(1)
    "  Ideally you will routinely run a fresh dbanalys at least weekly.       " skip
    skip(1)
   view-as
    alert-box question buttons yes-no
    title " Are You Sure? "
    update ok
  .

  if ok = yes then run makeRequest ( "xcmd=dba" ).

  return.

end.


procedure statementCache:

  define variable alertColor as integer no-undo.
  alertColor = getColorNum( "alert" ).

  csc_loop: do while true on error undo, leave
    on endkey undo, leave:

    update
      skip(1)
      '  The client statement cache is a powerful feature that sometimes causes  ' dcolor alertColor skip
      '  problems. If you are not comfortable with the potential issues, please  ' dcolor alertColor skip
      '  do not enable it in Production.                                         ' dcolor alertColor skip
      skip(1)
      '  Rather than globally enabling CSC, ProTop can enable or disable the     ' skip
      '  client statement cache for specific users by using the "#" command and  ' skip
      '  entering a usr#.                                                        ' skip
      skip(1)
      '  You can also use the menu at PROMON R&D, 1, 18 for fine-grained control ' skip
      '  over individual sessions.                                               ' skip
      skip(1)
      '  Global Client Statement Cache status:' CSC format "On/Off" '   On, Off, ? = no change  ' skip
      skip(1)
     with
      frame getCSC
      title " Global Client Statement Cache "
      row 3
      column ( if current-window:width-chars = 160 then 40 else 2 )
      width 78
      no-labels
      overlay
    .

    leave csc_loop.

  end.

  hide frame getCSC no-pause.

  if csc <> ? then
    run makeRequest ( substitute( "xcmd=csc_&1", string( CSC, "on/off" ))).

  return.

end.


procedure setRawMode:
  run makeRequest ( substitute( "xcmd=raw_&1",  entry( rawMode,  "Initial,Previous,Raw,Cumulative,Interval" ))).
  run makeRequest ( substitute( "xcmd=time_&1", entry( timeMode, "Summary,Rate" ))).
  return.
end.


procedure setRowLimit:
  run makeRequest ( substitute( "xcmd=rowLimit+&1", rowLimit )).
  return.
end.


procedure protopPause:
  pause.
  return.
end.


/* when a programmers is checking a bit of code in isolation it is
 * helpful to use this combination of settings
 */

procedure programmerMode:

  if programmerMode = no then
    do:
      assign
        sampleMode = "OnDemand"
        rawMode    = 5
        timeMode   = 1
      .
      run makeRequest ( substitute( "xcmd=raw_&1",  "Interval" )).
      run makeRequest ( substitute( "xcmd=time_&1", "Summary" )).
      programmerMode = yes.
    end.
   else
    do:
      assign
        sampleMode = "Auto"
        rawMode    = 5
        timeMode   = 2
      .
      run makeRequest ( substitute( "xcmd=raw_&1",  "Interval" )).
      run makeRequest ( substitute( "xcmd=time_&1", "Rate" )).
      programmerMode = no.
    end.

  return.

end.


/* use the profiler to help debug internal ProTop performance issues
 */

define variable zprofilerState as logical no-undo initial ?.

procedure zprofiler:

  /*** message "Profiler state:" zprofilerState. ***/

  if zprofilerState = yes then								/* the profiler is already running				*/
    do:

      run zprofiler_off.								/* flip the state of the profiler to "off"			*/
      zprofilerState = no.

      run zprofiler_load.								/* load profiler data into temp-tables to analyze		*/
      run zprofiler_proc.								/* process the data						*/
      run zprofiler_topx( no ).								/* report on the top X execution time lines -- to file		*/
      run zprofiler_topx( yes ).							/* report on the top X execution time lines -- to TTY		*/

      message zprofilerState "run zprofiler_off".

      return.										/* do not continue after this -- return				*/

    end.

  if zprofilerState = ? then								/* we need to launch the profiler				*/
    do:
      run lib/zprofiler.p persistent (							/* launch the PP						*/
        "protop3",									/* output file basename						*/
        "ProTop3 Execution Profile"							/* description							*/
      ).
    end.

  zprofilerState = no.									/* default to no						*/

  message
    color value( "red" )
    skip(1)
    "  The Profiler capability is used to track down performance issues   " skip
    "  within the ProTop client.  It is very unusual for an end-user to   " skip
    "  need to run this for that purpose.                                 " skip
    skip(1)
    "  Aside from debugging ProTop this code is also a useful example of  " skip
    "  embedding the profiler within an application.  The source can be   " skip
    "  found in lib/zprof*.p                                              " skip
    skip(1)
    "  It is fine to run this code in order to get a feel for how useful  " skip
    "  embedded profiling can be (IMHO it is *VERY* useful).              " skip
    skip(1)
    "  But be aware that profiling can very quickly create very large     " skip
    "  temp files (gigabytes in minutes) so do not run this just for      " skip
    "  giggles and do not leave it running unattended.                    " skip
    skip(1)
   view-as
    alert-box question buttons yes-no
    title " Are You Sure? "
    update zprofilerState
  .

  if zprofilerState = yes then
    do:

      run zprofiler_on.

      message
        color value( "red" )
        skip(1)
        '  The Profiler is now enabled.  Press "y"  ' skip    
        '  when you are ready to view the results.  ' skip   
        skip(1)
       view-as
        alert-box information
        title " Profiler Enabled "
      .

    end.

  return.

end.


procedure startWithProfilerOn:

  define variable ok as logical no-undo.

  message
    color value( "red" )
    skip(1)
    "  You are starting ProTop with an active external request to enable  " skip
    "  profiling via:                                                     " skip
    "    " prfFileName
    skip(1)
    "  But be aware that profiling can very quickly create very large     " skip
    "  temp files (gigabytes in minutes) so do not run this just for      " skip
    "  giggles and do not leave it running unattended.                    " skip
    skip(1)
    "  Do you really want to enable the profiler?                         " skip
    skip(1)
   view-as
    alert-box question buttons yes-no
    title " Are You Sure? "
    update ok
  .

  if ok = no then
    do:

      run lib/zprof_off.p.

      os-delete value( prfFileName ).

      message
        skip(1)
        '  The Profilng request has been disabled.  ' skip    
        skip(1)
       view-as
        alert-box information
        title " Profiler Disabled "
      .

    end.

end.


procedure demo0:									/* clear all running demo jobs					*/
  os-command silent value( "rm -f *.flg" ).
  return.
end.


procedure demo1:									/* fire up the demo suite					*/

  if lookup( ldbname(1), demoList ) < 1 then						/* only if this is a demo database!				*/
    do:
      message
        skip(1)
        '  ' ldbname(1) 'is not recognized as a "sports" variant.  ' skip
        skip(1)
       view-as
        alert-box information
        title " Demo Blended Workload "
      .
      return.
    end.

  run demoX ( "churn", "readalot" ).
  run demoX ( "churn", "chugger" ).
  run demoX ( "churn", "flail" ).
  run demoX ( "churn", "thrash" ).
  run demoX ( "churn", "puppy" ).

  run demoX ( "lockalot", "lockalot" ).
  run demoX ( "updorder", "updorder" ).
  run demoX ( "badtrx",   "badtrx" ).
  run demoX ( "blocker",  "blocker" ).
  run demoX ( "victim",   "victim" ).

  return.

end.


procedure demoX:									/* launch a demo program					*/

  define input parameter program as character no-undo.
  define input parameter uName   as character no-undo.

  define variable b_user as handle    no-undo.
  define variable ok     as logical   no-undo.
  define variable cmd    as character no-undo.
  define variable winPID as integer   no-undo.

  if lookup( ldbname(1), demoList ) < 1 then						/* only if this is a demo database!				*/
    do:
      message
        skip(1)
        '  ' ldbname(1) 'is not recognized as a "sports" variant.  ' skip
        skip(1)
       view-as
        alert-box information
        title " Demo Workload "
      .
      return.
    end.

  create buffer b_user for table "dictdb._User" no-error.
  ok = b_user:find-first( substitute( 'where _userId = "&1"', uName ), no-lock ) no-error.

  if opsys = "unix" then
    do:
      if ok = yes then
        cmd = substitute( '_progres -b &1 -p s2k_demo/&2.p -rand 2 -T &3 -clientlog &3~\&2.debug -U &4 -P &5 > &4.log 2>&&1 &&', pdbname(1), program, pt_tmpdir, uName, "abc123" ). 
       else
        cmd = substitute( '_progres -b &1 -p s2k_demo/&2.p -rand 2 -T &3 -clientlog &3~\&2.debug > &2.log 2>&&1 &&', pdbname(1), program, pt_tmpdir ). 
      os-command silent value( cmd ).
    end.
   else
    do:
      if ok = yes then
        cmd = substitute( '_progres -b &1 -p s2k_demo~\&2.p -rand 2 -T &3 -basekey "INI" -ininame etc~\protop.ini -clientlog &3~\&2.debug -U &4 -P &5', pdbname(1), program, pt_tmpdir, uName, "abc123" ). 
       else
        cmd = substitute( '_progres -b &1 -p s2k_demo~\&2.p -rand 2 -T &3 -basekey "INI" -ininame etc~\protop.ini -clientlog &3~\&2.debug', pdbname(1), program, pt_tmpdir ). 
      run spawn( cmd, "", output winPID ).
    end.

  delete object b_user.

  return.


end.


procedure userDefined:
  
  define variable p as handle no-undo. 

  define variable isRunning as logical no-undo.

  if lookup( ldbname(1), demoList ) < 1 then						/* only if this is a demo database!				*/
    do:
      message
        skip(1)
        '  ' ldbname(1) 'is not recognized as a "sports" variant.  ' skip
        skip(1)
       view-as
        alert-box information
        title " User Defined Metrics Demo "
      .
      return.
    end.

  p = session:first-procedure.								/* first *persistent* procedure					*/
  do while valid-handle( p ) and isRunning = no:
    if p:file-name matches "*lib/sports2000*" then
      isRunning = yes.
    p = p:next-sibling.
  end.

  if isRunning = yes then
    do:
      message
        skip(1)
        '  lib/sports2000.p is already running persistently.  ' skip
        skip(1)
       view-as
        alert-box information
        title " User Defined Metrics Demo "
      .
    end.
   else
    do:
      run value( "lib/sports2000.p" ) persistent.					/* contains code that is tied to the sports2000 db		*/
      message
        skip(1)
        '  lib/sports2000.p has been launched persistently.  ' skip
        skip(1)
       view-as
        alert-box information
        title " User Defined Metrics Demo "
      .
    end.

end.


procedure dotCommand:									/* semi-hidden menu of demo features and esoterica		*/

  define input parameter c as character no-undo case-sensitive.

  case c:

    when "." then
     do:
       if       dotMode = 0 then dotMode = 1.						/* this shouldn't happen					*/
        else if dotMode = 1 then dotMode = 2.						/* make dot mode sticky						*/
        else if dotMode = 2 then dotMode = 0.						/* exit sticky dot mode						*/
       return.
     end.

    when "0" then run demo0.								/* shutdown running demos					*/
    when "1" then run demo1.								/* launch the whole suite					*/

    when "2" then run demoX ( "churn", entry( random( 1, 5 ), "readalot,chugger,flail,thrash,puppy" )).
    when "3" then run demoX ( "lockalot", "lockalot" ).
    when "4" then run demoX ( "updorder", "updorder" ).
    when "5" then run demoX ( "badtrx",   "badtrx" ).
    when "6" then run demoX ( "blocker",  "blocker" ).
    when "7" then run demoX ( "victim",   "victim" ).
    when "8" then run demoX ( "badorder", "badorder" ).

    when "u" then run userDefined.							/* user defined metrics demo					*/

    when "&" then publish "zecret_properties".						/* show some exotic, semi-secret properties			*/

    when "h" or when "?" then publish "protop_dothelp".					/* help for the dot menu					*/

    {ssg/sausage04.i}

  end.

  if dotMode <= 1 then
    dotMode = 0.									/* exit non-sticky dot mode					*/

  return.

end.


/* the contents of the CASE statement could probably be managed dynamically
 * by adding the keystroke mapping to the viewer/browser initialization
 *
 * that could also be used to build the help screen contents if one were
 * so inclined
 */

procedure command:

  define input parameter c as character no-undo case-sensitive.

  define variable k as character no-undo.

  define variable h as integer   no-undo.
  define variable w as integer   no-undo.

  if dotMode >= 1 then
    run dotCommand( c ).
   else
    do:

      case c:

        when "a" then publish "protop_show" ( "StorageAreas" ).
        when "b" then publish "protop_show" ( "Blocked" ).
        when "c" then publish "protop_show" ( "Configuration" ).
        when "d" then publish "protop_show" ( "Dashboard" ).
        when "e" then publish "protop_show" ( "appActivity" ).				/* aka appmon							*/
        when "f" then publish "protop_show" ( "FileIOActivity" ).
        when "g" then publish "notyet" ( c ).						/* available							*/
        when "h" or when "?" then
          do:
            publish "protop_help".
            k = keylabel( lastkey ).
            if k <> "h" and k <> "?" then run command( k ).
          end.
        when "i" then publish "protop_show" ( "IndexActivity" ).
        when "j" then publish "protop_show" ( "pasoeInfo" ).
        when "k" then publish "protop_show" ( "Checkpoints" ).
        when "l" then
          do:
            if integer( entry( 1, proversion, "." )) >= 12 then
              publish "protop_show" ( "LOBActivity" ).
             else
              publish "protop_noLOBs".
          end.
        when "m" then publish "protop_show" ( "TenantInfo" ).
        when "n" then publish "notyet" ( c ).						/* available							*/
        when "o" then publish "protop_show" ( "OSInfo" ).
        when "p" then run protopPause.
        when "q" then publish "protop_quit".
        when "r" then publish "protop_show" ( "ReplAgent" ).
        when "s" then publish "protop_show" ( "RemoteServerActivity" ).
        when "t" then publish "protop_show" ( "TableActivity" ).
        when "u" then publish "protop_show" ( "UserIOActivity" ).
        when "v" then run viewer.
        when "w" then publish "protop_show" ( "LatchActivity" ).
        when "x" then publish "protop_show" ( "ActiveTRX" ).
        when "y" then run zprofiler.
        when "z" then publish "notyet" ( c ).
  
        when "A" then publish "protop_show" ( "AppSrvStatus" ).
        when "B" then publish "protop_show" ( "BigBGuesstimator" ).
        when "C" then publish "protop_show" ( "CDCQ" ).
        when "D" then publish "protop_show" ( "df" ).
        when "E" then publish "notyet" ( c ).						/* available							*/
        when "F" then publish "notyet" ( c ).						/* available							*/
        when "G" then publish "notyet" ( c ).						/* available							*/
        when "H" then run util/dohc.p.
        when "I" then publish "notyet" ( c ).						/* available							*/
        when "J" then publish "notyet" ( c ).						/* available							*/
        when "K" then publish "notyet" ( c ).						/* available							*/
        when "L" then publish "protop_show" ( "BrokerConfig" ).
        when "M" then publish "notyet" ( c ).						/* available							*/
        when "N" then publish "protop_show" ( "netstat" ).
        when "O" then publish "protop_show" ( "OSInfo" ).				/* duplicate of "o" (deprecated)				*/
        when "P" then publish "protop_getUserPID" ( input-output userPID, input-output userNum ).
        when "Q" then publish "protop_show" ( "SQLActivity" ).
        when "R" then run util/dbparamrpt.p.						/* db parameter report						*/
        when "S" then run util/serverrpt.p.
        when "T" then publish "protop_showRangeData".
        when "U" then publish "protop_show" ( "UserInformation" ).
        when "V" then publish "notyet" ( c ).						/* available							*/
        when "W" then publish "protop_show" ( "Who" ).
        when "X" then publish "notyet" ( c ).						/* available							*/
        when "Y" then run lib/showustats.p.
        when "Z" then
          do:

            OS-GetWinSize( input-output numRows, input-output numColumns ).		/* has the window size changed?					*/

            if opsys = "unix" then							/* terminal = terminal does nothing on Windows :(		*/
              terminal = terminal.							/* re-reads protermcap and reinitializes the terminal,		*/
											/* among other things that will reset screen-lines		*/

            run protop_display(  "", ? ).						/* display nothing but recalculate layout			*/
            publish "protop_refresh".							/* refresh the screen!						*/

          end.

        when "0" then publish "protop_colors".						/* show the protop color table					*/
    /*  when "0" then publish "notyet" ( c ).	*/					/* available							*/
        when "1" then run util/dbparamrpt.p.						/* db parameter report						*/
        when "2" then publish "protop_show" ( "Pro2Queues" ).				/* looking for a new home					*/
        when "3" then publish "protop_setConnectFilter".				/* set the connection type filter for UIO			*/
        when "4" then publish "protop_show" ( "svcTelemetry" ).
        when "5" then brSP = ( if brSP = 0 then 1 else 0 ).
        when "6" then publish "protop_getUserTblName" ( input-output userTblName, input-output userTblNum ).
        when "7" then publish "protop_getUserIdxName" ( input-output userIdxName, input-output userIdxNum ).
        when "8" then publish "protop_show" ( "UsersOfTable" ).
        when "9" then publish "protop_show" ( "UsersOfIndex" ).

        when "+" then publish "notyet" ( c ).						/* available							*/
        when "-" then publish "notyet" ( c ).						/* available							*/

        /* some of the following keys might be problematic; especially from an i18n perspective
         * OTOH they are pretty much required in order to write code so it is mostly a matter of
         * keyboard contortions
         */

        when "!" then publish "protop_dbanalys".
        when "@" then run protopMail.
        when "#" then publish "protop_getUserNum" ( input-output userNum, input-output userPID ).
        when "$" then publish "protop_setSampleMode".
        when "%" then
         do:
           publish "protop_getRawMode" ( input-output rawMode, input-output timeMode ).
           publish "protop_setRawMode".							/* output needs to be returned before publishing...		*/
         end.
        when "^" then run protopSort.
        when "&" then publish "protop_properties".
        when "*" then publish "protop_statementCache".
        when "(" then publish "protop_getRowLimit" ( input-output rowLimit ).
        when ")" then publish "protop_getLockLimit" ( input-output pt_lktbllim ).
        when "_" then publish "protop_clearCSC".					/* if this changes you also need to edit lib/setvardb.p		*/
        when "=" then publish "notyet" ( c ).						/* available							*/

        when "." then publish "dotMode".
        when "," then publish "notyet" ( c ).						/* available							*/
        when ":" then publish "notyet" ( c ).						/* available							*/
        when ";" then publish "protop_getMonInt"  ( input-output monInt ).
        when "'" then publish "notyet" ( c ).						/* available							*/
        when '"' then publish "notyet" ( c ).						/* available							*/
        when ">" then publish "notyet" ( c ).						/* available							*/
        when "<" then publish "notyet" ( c ).						/* available							*/

        when "["  then									/* light mode							*/
          do:
            /* if terminal begins "xterm" then */
            terminal = "light".
          end.

        when "]"  then									/* dark mode							*/
          do:
            /* if terminal begins "xterm" then */
            terminal = "dark".
          end.

        when "~{" then
          do:
            run lib/bgcolorpicker.p.							/* pick a background color					*/
          end.


        when "~}" then publish "notyet" ( c ).						/* available							*/
        when "~~" then run lib/clife.p.
        when "`"  then publish "notyet" ( c ).						/* available							*/
        when "~\" then pt_showInterest = not( pt_showInterest ).
        when "|"  then publish "notyet" ( c ).						/* available							*/
        when "/"  then publish "protop_show" ( "SequenceActivity" ).

/*      when "?"	*/								/* see "h"							*/

        when " " then .

      end.	/* case */

  end.

  return.

end.


on ctrl-a of nothing in frame notHere do:
  something = something + "^a".
  publish "protop_show" ( "aiinfo" ).
  return.
end.

on ctrl-b of nothing in frame notHere do:
  something = something + "^b".
  publish "protop_show" ( "b2" ).
  return.
end.

on ctrl-c of nothing in frame notHere do:						/* not really available, coded for completeness			*/
  something = something + "^c".
  publish "notyet" ( "^c" ).
  return.
end.

on ctrl-d of nothing in frame notHere do:
  something = something + "^d".
  publish "dump_load".
  return.
end.

/* do not capture this -- ^X is an alternative to F1 or GO in user dialogs
 *
 * on ctrl-e of nothing in frame notHere do:						/* not really available, coded for completeness			*/
 *   something = something + "^e".
 *   publish "notyet" ( "^e" ).								/* equivalent to F4 or END-ERROR				*/
 *   return.
 * end.
 *
 */

on ctrl-f of nothing in frame notHere do:
  something = something + "^f".
  publish "protop_faqhelp".
  return.
end.

on ctrl-g of nothing in frame notHere do:
  something = something + "^g".
  publish "notyet" ( "^g" ).								/* available							*/
  return.
end.

on ctrl-h of nothing in frame notHere do:
  something = something + "^h".
  publish "notyet" ( "^h" ).								/* available							*/
  return.
end.

on ctrl-i of nothing in frame notHere do:
  something = something + "^i".
  publish "notyet" ( "^i" ).								/* available							*/
  return.
end.

on ctrl-j of nothing in frame notHere do:
  something = something + "^j".
  publish "notyet" ( "^j" ).								/* available							*/
  return.
end.

on ctrl-k of nothing in frame notHere do:
  something = something + "^k".
  publish "notyet" ( "^k" ).								/* available							*/
  return.
end.

on ctrl-l of nothing in frame notHere do:
  something = something + "^l".
  publish "protop_show" ( "licenseCfg" ).
  return.
end.

on ctrl-m of nothing in frame notHere do:						/* people get confused and annoyed by the notyet msg here	*/
/*
 * something = something + "^m".
 * publish "notyet" ( "^m" ).								/* available							*/
 */
  return.
end.

on ctrl-n of nothing in frame notHere do:
  something = something + "^n".
  publish "notyet" ( "^n" ).								/* available							*/
  return.
end.

on ctrl-o of nothing in frame notHere do:
  something = something + "^o".
  publish "notyet" ( "^o" ).								/* available							*/
  return.
end.

on ctrl-p of nothing in frame notHere do:
  something = something + "^p".
  run programmerMode.
  return.
end.

on ctrl-q of nothing in frame notHere do:						/* not really available, coded for completeness	*/
  something = something + "^q".
  publish "notyet" ( "^q" ).								/* used for flow control on UNIX				*/
  return.
end.

on ctrl-r of nothing in frame notHere do:
  something = something + "^r".
  publish "dba_report".
  return. /* no-apply. */
end.

on ctrl-s of nothing in frame notHere do:						/* not really available, coded for completeness			*/
  something = something + "^s".
  publish "notyet" ( "^s" ).								/* used for flow control on UNIX				*/
  return.
end.

on ctrl-t of nothing in frame notHere do:
  something = something + "^t".
  if os-getenv( "TTDEBUG" ) = "yes" then
    publish "protop_showTT" ( "" ).
   else
    do:
      message
        skip(1)
        '  Temp table statistics require OpenEdge 11 or higher  ' skip
        '  and must be enabled by setting:                      ' skip
        skip(1)
        '        export TTDEBUG=yes                             ' skip
        skip(1)
        '  in $PROTOP/bin/localenv                              ' skip
        skip(1)
        '  You must also uncomment the -tt* parameters in       ' skip
        '  etc/protop.pf:                                       ' skip
        skip(1)
        '      -ttbaseindex 1                                   ' skip
        '      -ttbasetable 1                                   ' skip
        '      -ttindexrangesize 1000                           ' skip
        '      -tttablerangesize 1000                           ' skip
        skip(1)
       view-as alert-box information title " Temp Table Statistics "
      .
    end.
  return.
end.

on ctrl-u of nothing in frame notHere do:
  something = something + "^u".
  run util/genupdsql.p.
  return.
end.

on ctrl-v of nothing in frame notHere do:
  something = something + "^v".
/*  publish "notyet" ( "^v" ).	*/							/* available							*/
  run viewer.
  return.
end.

on ctrl-w of nothing in frame notHere do:
  something = something + "^w".
  publish "notyet" ( "^w" ).								/* available							*/
  return.
end.

/* do not capture this -- ^X is an alternative to F1 or GO in user dialogs
 *
 * on ctrl-x of nothing in frame notHere do:						/* not really available, coded for completeness			*/
 *   something = something + "^x".
 *   publish "notyet" ( "^x" ).								/* F1 or GO							*/
 *   return.
 * end.
 *
 */

on ctrl-y of nothing in frame notHere do:
  something = something + "^y".
  publish "notyet" ( "^y" ).								/* available							*/
  return.
end.

on ctrl-z of nothing in frame notHere do:
  something = something + "^z".
  publish "notyet" ( "^z" ).								/* available							*/
  return.
end.

on esc-d OF nothing in frame notHere do:
  something = something + "esc-d".
  publish "protop_getDbgMode".
  return.
end.

{ssg/sausage02.i}


procedure clearRedArea:
  /* display something dcolor 6 with frame notHere. */					/* 6 is green	*/
  display something dcolor 7 with frame notHere.					/* 7 is blue	*/
  return.
end.


/* initialization
 *
 */

if num-entries( session:parameter, "|" ) >= 1 then pt_shortname = entry( 1, session:parameter, "|" ).
if num-entries( session:parameter, "|" ) >= 2 then ptStartup    = entry( 2, session:parameter, "|" ).

if num-entries( session:parameter, "|" ) >= 3 then ptParams     = entry( 3, session:parameter, "|" ).		/* currently unused			*/
if num-entries( session:parameter, "|" ) >= 4 then ptCnxStr     = entry( 4, session:parameter, "|" ).		/* currently unused			*/

if session:window-system <> "tty" then							/* try to look reasonable if someone fires up prowin		*/
  assign
    current-window:hidden       = true
    session:v6display           = true
    current-window:height-chars = 60
    current-window:width-chars  = 162
    current-window:bgcolor      = 15
    current-window:hidden       = false
    current-window:visible      = true
  .

/* current-window:message-area = no.							/* no such luck :(						*/
 * current-window:status-area  = no.
 */

/* run lib/chkscreen.p.	*/								/* is the screen big enough?					*/

connectMode = "Direct".

/* subscribe to "logMsg"                anywhere run-procedure "logMsg". */		/* obsolete	*/

subscribe to "notyet"                anywhere run-procedure "notyet".
subscribe to "dotMode"               anywhere run-procedure "dotMode".
subscribe to "protop_quit"           anywhere run-procedure "quitme".
subscribe to "protop_command"        anywhere run-procedure "command".
subscribe to "protop_refresh"        anywhere run-procedure "refresh".
subscribe to "protop_register"       anywhere run-procedure "register".
subscribe to "protop_setRawMode"     anywhere run-procedure "setRawMode".
subscribe to "protop_setRowLimit"    anywhere run-procedure "setRowLimit".
subscribe to "protop_setSampleMode"  anywhere run-procedure "setSampleMode".
subscribe to "protop_userinfo2"      anywhere run-procedure "userInfo2".
subscribe to "protop_statementCache" anywhere run-procedure "statementCache".
subscribe to "protop_dbanalys"       anywhere run-procedure "dbanalys".

subscribe to "clearRedArea"          anywhere run-procedure "clearRedArea".

run lib/chkbg.p.

run lib/ptheader.p ( input-output headerFrame, input-output modeIndicator, input-output protopVersion, input-output clockString ).

if fast-start then pause 0 before-hide.							/* set fast-start = no to preserve startup messages		*/

message "Starting ProTop...".

run lib/protoplib.p  persistent.							/* load protop infrastructure library           		*/

if connected( "dictdb" ) then								/* needs to be loaded prior to protop-cfg because		*/
  do:											/* isReplTarget() is potentially called by setptname		*/
    message "Loading vstlib...".
    run lib/vstlib.p persistent.							/* load vst function library                    		*/
  end.

run lib/protop-cfg.p persistent.							/* configure the protop environment                		*/

file-info:file-name = pt_tmpdir.							/* make certain that we have a temp directory!			*/
if file-info:full-pathname = ? then
  os-command silent value( "mkdir " + pt_tmpdir ).

file-info:file-name = pt_logdir.							/* make certain that we have a log directory!			*/
if file-info:full-pathname = ? then
  os-command silent value( "mkdir " + pt_logdir ).

dbgFileName = substitute( "&1/&2.&3.&4", pt_tmpdir, monName, ( if pt_shortname <> "" then pt_shortname else ldbname(1)), "dbg" ).
prfFileName = substitute( "&1/&2.&3.&4", pt_tmpdir, monName, ( if pt_shortname <> "" then pt_shortname else ldbname(1)), "prf.flg" ).

run chkDbgMode( dbgFileName, input-output dbgMode ).					/* chkDbgMode() is in lib/protoplib.p				*/

run chkProfiler( prfFileName, input-output prfMode ).					/* perhaps we are being asked to turn the profiler on?		*/

if prfMode = "on" then									/* it is _probably_ a mistake if this happens			*/
  run startWithProfilerOn.

if not connected( "dictdb" ) then
  do:
    ptStartup = "osinfo".
  end.
 else
  do:

    run lib/ptsetvardb.p persistent.							/* set variables that affect behavior and need a cnx		*/

    message "Checking table and index ranges...".
    run lib/xrange.p persistent.							/* if necessary warn about table and index range sizes		*/

    message "Initializing _userTable and _userIndex stat collection for this session...".

    run lib/usertablestats.p persistent.						/* some additional useful debugging support			*/

    if screen-lines < 30 then
      ptStartup = "Dashboard,UserIOActivity".
     else
      ptStartup = "Dashboard,TableActivity,IndexActivity,UserIOActivity".

    message "Startup data collectors:" ptStartup.

  end.

if session:batch = no and current-window:width-chars < 160 then
  assign
    pt_altWidth     = "80"
    pt_showInterest = yes
  .

run lib/dynscreen.p persistent.								/* dynamic screen painting routines				*/
run lib/oi.p        persistent.								/* "of interest"						*/
run lib/ptcolor.p   persistent.								/* protop color table						*/

assign
  dataColor  = getColorNum( "data" )
  goodColor  = getColorNum( "good" )
  alertColor = getColorNum( "alert" )
.

if session:window-system = "tty" then
  run lib/ptsplash.p.
 else
  .											/* we should do something with the fancy logo!			*/

if connected( "dictdb" ) then run lib/chkcs.p.						/* are we running in client/server mode?			*/

if os-getenv( "TTDEBUG" ) = "yes" then
  run lib/ttinfo.p persistent.								/* some useful debugging support				*/

run ssg/sausage42.p persistent.

{ssg/sausage05.i}

run lib/mailx.p     persistent.								/* handle mail							*/
run lib/ptsetvar.p  persistent.								/* set variables that affect protop.p behavior			*/
run lib/pthelp.p    persistent.								/* the help screen						*/
run lib/ptprops.p   persistent.								/* properties controlled by environment variables		*/

run ssg/sausage02.p persistent.

publish "protop_checkRangeData".

run lib/jsonlib.p persistent.

message "Launching data collectors...".

if connectMode = "direct" then
  do:
    run lib/ptsrv.p persistent.								/* launch data collectors in the srv directory			*/
    run initProTopServer( "direct", "", "" ).						/* no arguments needed for direct connections			*/
  end.

if dbgMode >= 3 then
  message "Setting up data collector UI...".

run getConfig( "protop.json", output result ).						/* find protop.json (describes data collectors)			*/
if result begins '~{"tt_protop":[' then
  do:

    result = replace( result, "~n", "" ).						/* pretty printed json is too challenging for json2tt		*/
    result = trim( substring( result, index( result, "[" ) + 1 ), "~{~}]" ).		/* and we just want the rows - none of the wrapper		*/

    run json2tt( string( result ), ( buffer tt_protop:handle )).

  end.

result = "".

/* create the UI objects (lib/viewer.p, lib/browser.p) for each data collector
 * and spit out a report to pt_tmpdir in case something goes wrong
 */

output stream initStrm to value( substitute( "&1/pt3.&2.init", pt_tmpdir, pt_shortName )).

put stream initStrm unformatted now " ProTop install dir = " os-getenv( "PROTOP" ) " version = [{lib/ptversion.i}]" " asof = [{lib/asof.i}]" " build = [{lib/commit.i}]" skip.
put stream initStrm unformatted now " DLC = " os-getenv( "DLC" ) " proversion = " proversion skip.
put stream initStrm unformatted now " propath = " propath skip.
put stream initStrm unformatted now " dbgMode: " dbgMode skip.

if connected( "dictdb" ) then
  do:
    put stream initStrm unformatted now " connected to: " pdbname(1) skip.
  end.
 else
  do:
    put stream initStrm unformatted now " no database connection" skip.
  end.

put stream initStrm unformatted skip(1).

put stream initStrm "Date (YMD)               Time  Order Type       TT Name          Procedure Name       Data Collector Name  Screen Title" skip.
put stream initStrm "========== ==================  ===== ========== ================ ==================== ==================== ==============================" skip.

for each tt_protop by tt_protop.order:

  /* Create instances of lib/browser.p (browse panels), lib/viewer.p ("viewers"), and lib/info.p (additional title bar info).
   * But we don't want to create them for the PASOE data collector if we aren't running OE 11.7+
   */
  if tt_protop.dcName = "pasoeInfo" and
     decimal( right-trim( proversion, "abcdefghijklmnopqrstuvwxyz" )) < 11.7 then next.

  run value( substitute( "lib/&1.p", scrType )) persistent ( tt_protop.order, scrTitle, dcName, ttName, minRows, maxRows, ptCmd ).

  put stream initStrm
    now                     space(2)
    order    format ">>>>9" space
    scrType  format "x(10)" space
    ttName   format "x(16)" space
    procName format "x(20)" space
    dcName   format "x(20)" space
    scrTitle format "x(30)" space
    skip
  .

end.

output stream initStrm close.

/* ok, we're ready for business!
 */

if dbgMode >= 3 then
  message "Initializing header...".

if pdbname( 1 ) <> ? then								/* check to see if a new DBAnalys is available			*/
  run checkDBX.

publish "protop_dbid".									/* line #2 of the header - dbId data collector			*/

if index( ptStartup, "dbid" ) <= 0 then							/* the dbid data collector must always run			*/
  ptStartup = "dbId," + ptStartup.

ptStartup = right-trim( ptStartup, "," ).

do i = 1 to num-entries( ptStartup ):
  publish "protop_show" ( entry( i, ptStartup )).					/* startup with the specified data collectors visible		*/
end.

if lookup( ldbname(1), demoList ) > 0 then
  run value( "lib/sports2000.p" ) persistent.						/* contains demo code that is tied to the sports2000 db		*/

if fast-start = no or dbgMode >= 3 then							/* set fast-start = no to preserve startup messages		*/
  pause.										/* slow start, pause to read startup messages			*/

hide message no-pause.

if session:batch = no then
  do:
    run lib/osgetwinsize.p persistent.
    OS-GetWinSize( input-output numRows, input-output numColumns ).			/* initialize window size					*/
  end.

xtraStart = os-getenv( "XTRASTART" ).							/* extra startup commands as keystrokes				*/
do i = 1 to length( xtraStart ):
  publish "protop_command" ( substring( xtraStart, i, 1 )).
end.

publish "protop_getReqStr" ( input-output initReqStr ).					/* only request the data collectors which are visible		*/

run makeRequest( initReqStr ).

numSamples = numSamples + 1.

lastRefresh = time.									/* avoiding datetime-tz to preserve possible v9 compatibility	*/
publish "protop_refresh".

/* Prior to OE 11.2, PAUSE values are integers; fractional values are syntactically valid but are rounded to integers.
 * In 11.2+, the AVM's internal datatype changes to decimal. This means v11 r-code compiled with 11.2+ but run in an AVM
 * prior to 11.2 will interpret a short pause, e.g. 1 second, as an extremely long pause. 
 *
 * More info: https://community.progress.com/s/article/000045979
 *
 * As we build our v11 r-code in 11.7, such clients will have to do session compiles (i.e. remove protop-v11.pl from the 
 * propath, by setting PROPATH=${PROTOP} in bin/localenv, and use the encrypted source).
 *
 * The proversion value may end with letters, e.g. 10.2B, 11.2ALPHA, 12.3BETA, etc.; we remove those to permit comparison.
 */

if decimal( right-trim( proversion, "abcdefghijklmnopqrstuvwxyz" )) > 11.1 then
  tick = 0.25.                      /* 0.1 is too fast, 0.5 is a bit slow (choppy screens) */
 else
  tick = 1.                         /* OE 11.1 and earlier only take integer values */

define variable keyPressed as logical no-undo.
define variable keyTimeOut as logical no-undo.

if session:batch = no then
  enable nothing with frame notHere.

something = "Z".


/* main block
 *
 */

main_block: do while keepRunning on error undo, retry:

  if pt_sampleLimit > 0 and numSamples >= pt_sampleLimit then
    do:
      message now "numSamples:" numSamples ">=" pt_sampleLimit.
      if session:batch = no then pause.							/* don't cover up the screen with an alert-box			*/
      leave.										/* but we do need to pause because the screen going away	*/
    end.

/* The "status line" is the first 62 characters on the last line of the screen:                         */
/*                                                                                                 	*/
/*                         1         2         3         4         5         6         7         8 	*/
/*                12345678901234567890123456789012345678901234567890123456789012X45678901234567890 	*/
/*                |                                                            |                  	*/
/*                v                                                            v                  	*/

  status default "? = help  q = quit  p = pause  <space> = refresh  ^r = reports".
  status input   "? = help  q = quit  p = pause  <space> = refresh  ^r = reports".

/*                |<---------------------------------------------------------->|                  	*/

  display something dcolor goodColor with frame notHere.				/* typing while green works					*/

  /*** if keystrokes mysteriously vanish it is likely that some code somewhere
   *** is doing INPUT FROM or INPUT THROUGH without a STREAM -- track down that
   *** code and add a STREAM
   ***
   ***/

  /***/ run ofInterest. /***/

  assign
    keyPressed = no
    keyTimeOut = no
  .

  do while keyTimeOut = no:								/* keys typed rapidly execute without intervening refreshes	*/

    apply "entry" to nothing in frame notHere.						/* steal focus back if someone gets all clicky			*/

    if session:batch = no then
      readkey pause tick.								/* PAUSE can take fractions starting with OE 11.2			*/
     else
      pause tick.

    if lastkey < 0 then									/* timeout							*/
      keyTimeOut = yes.
     else										/* keystroke!							*/
      do:
        keyPressed = yes.
        if keyfunction( lastkey ) = "end-error" then					/* abort!							*/
          leave main_block.
        display something dcolor dataColor with frame notHere.				/* show that stuff is updating					*/
        apply lastkey to nothing in frame notHere.					/* do some real work :)						*/
      end.

  end.

  /*  we have waited long enough -or- something was typed
   */

  if keyPressed = yes or
     ( sampleMode = "Auto" and abs( time - lastRefresh ) >= monint ) then		/* abs() handles rolling over midnite				*/
    do:

      display something dcolor alertColor with frame notHere.				/* show that stuff is updating					*/

      if keyPressed = no then								/* only do this stuff when otherwise idle			*/
        do:

          run chkDbgMode( dbgFileName, input-output dbgMode ).				/* debug mode might have been changed externally		*/
          run chkProfiler( prfFileName, input-output prfMode ).				/* perhaps we are being asked to turn the profiler on or off?	*/
          if pdbname( 1 ) <> ? then run checkDBX.					/* dbanalys may have been updated				*/

          /* has the window size changed?
           */

          if pt_autoresize = yes and OS-GetWinSize( input-output numRows, input-output numColumns ) then
            do:
              terminal = terminal.							/* re-initialize from protermcap (invokes "ws" etc)		*/
              run protop_display( "", ? ).						/* display nothing but recalculates layout			*/
              /* publish "protop_refresh".	*/					/* refresh the screen!						*/
              if dbgMode >= 1 then
                message numRows "rows" numColumns "columns".
            end.

        end.

      lastRefresh = time.								/* lastRefresh = now. is a needless use of datetime		*/
      publish "protop_refresh".								/* refresh the screen!						*/

      display something dcolor goodColor with frame notHere.				/* show that stuff is updating					*/

      numSamples = numSamples + 1.

    end.

end.

end.	/* on stop etc ... */

publish "protop_clearCSC".								/* if the user set any csc options offer to clear them		*/

run lib/zprof_off.p.									/* turn profiling off in order to get output (just in case)	*/

file-info:file-name = prfFileName.
if file-info:full-pathname <> ? then
  do:
    message now "deleting: " prfFileName.
    os-delete value( file-info:full-pathname ).
  end.

if program-name( 4 ) <> ? then								/* was protop started via -p?					*/
  return.										/* someone called us so return there				*/
 else
  quit.											/* do not drop out to the "tram lines"				*/

/* the end */
